"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const rt = require("./request-type");
/**
 * registerMasterHandler registers a language server handler on the
 * main connection (first parameter) that delegates work to two
 * workers (accessible through the connections named by parameters one
 * and two). Worker one is the "canonical" worker. It receives all
 * requests sent to the master. Worker two is an auxiliary, used to
 * speed up certain operations that we would not like to block on
 * other operations. (Due to the constraints of the Node.js runtime,
 * the master and each worker has only one thread of execution.)
 *
 * On hover and definition requests, the master handler will return
 * the first successful response from either of the two workers. On
 * symbol and references requests, the master handler will only
 * delegate to worker one. On initialize, the master handler forwards
 * the request to both workers, but only returns the response from
 * worker one. All notifications are forwarded to both workers.
 *
 * @param master       The connection to register on
 * @param leigthWeight Connection for short-running requests
 * @param heavyDuty    Connection for long-running requests
 */
function registerMasterHandler(master, lightWeight, heavyDuty) {
    // Forward calls from the worker to the master
    for (const worker of [lightWeight, heavyDuty]) {
        worker.onRequest(rt.ReadFileRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
            return master.sendRequest(rt.ReadFileRequest.type, params);
        }));
        worker.onRequest(rt.ReadDirRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
            return master.sendRequest(rt.ReadDirRequest.type, params);
        }));
    }
    // Forward initialize, shutdown
    master.onRequest(rt.InitializeRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        const [result] = yield Promise.all([
            lightWeight.sendRequest(rt.InitializeRequest.type, params),
            heavyDuty.sendRequest(rt.InitializeRequest.type, params)
        ]);
        return result;
    }));
    master.onRequest({ method: 'shutdown' }, () => __awaiter(this, void 0, void 0, function* () {
        const [result] = yield Promise.all([lightWeight, heavyDuty].map(worker => worker.sendRequest(rt.ShutdownRequest.type)));
        // Shutting down the master means killing the workers
        for (const worker of [lightWeight, heavyDuty]) {
            worker.sendNotification(rt.ExitRequest.type);
        }
        return result;
    }));
    // Notifications (both workers)
    master.onDidOpenTextDocument((params) => {
        for (const worker of [lightWeight, heavyDuty]) {
            worker.sendNotification(rt.TextDocumentDidOpenNotification.type, params);
        }
    });
    master.onDidChangeTextDocument((params) => {
        for (const worker of [lightWeight, heavyDuty]) {
            worker.sendNotification(rt.TextDocumentDidChangeNotification.type, params);
        }
    });
    master.onDidSaveTextDocument((params) => {
        for (const worker of [lightWeight, heavyDuty]) {
            worker.sendNotification(rt.TextDocumentDidSaveNotification.type, params);
        }
    });
    master.onDidCloseTextDocument((params) => {
        for (const worker of [lightWeight, heavyDuty]) {
            worker.sendNotification(rt.TextDocumentDidCloseNotification.type, params);
        }
    });
    // Short-running requests (worker one)
    // These are all that can typically only need very specific files to be parsed
    master.onDefinition((params) => __awaiter(this, void 0, void 0, function* () {
        return lightWeight.sendRequest(rt.DefinitionRequest.type, params);
    }));
    master.onHover((params) => __awaiter(this, void 0, void 0, function* () {
        return lightWeight.sendRequest(rt.HoverRequest.type, params);
    }));
    // Long-running requests (worker two)
    // These are all that require compilation of the full workspace
    master.onRequest(rt.WorkspaceSymbolsRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        return heavyDuty.sendRequest(rt.WorkspaceSymbolsRequest.type, params);
    }));
    master.onRequest(rt.DocumentSymbolRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        return heavyDuty.sendRequest(rt.DocumentSymbolRequest.type, params);
    }));
    master.onRequest(rt.WorkspaceReferenceRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        return heavyDuty.sendRequest(rt.WorkspaceReferenceRequest.type, params);
    }));
    master.onRequest(rt.DependenciesRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        return heavyDuty.sendRequest(rt.DependenciesRequest.type, params);
    }));
    master.onRequest(rt.XdefinitionRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        return heavyDuty.sendRequest(rt.XdefinitionRequest.type, params);
    }));
    master.onRequest(rt.GlobalRefsRequest.type, (params) => __awaiter(this, void 0, void 0, function* () {
        return heavyDuty.sendRequest(rt.GlobalRefsRequest.type, params);
    }));
    master.onReferences((params) => __awaiter(this, void 0, void 0, function* () {
        return heavyDuty.sendRequest(rt.ReferencesRequest.type, params);
    }));
    master.onCompletion((params) => __awaiter(this, void 0, void 0, function* () {
        return heavyDuty.sendRequest(rt.TextDocumentCompletionRequest.type, params);
    }));
}
exports.registerMasterHandler = registerMasterHandler;
//# sourceMappingURL=master-connection.js.map